import IKTransaction from './BaseEvent/IKTransaction';
import IKResponse, { IKBadRequestResponse, IKSuccessResponse } from './API/IKResponse';
import IKGlobals from './IKGlobals';
//
export default class IKEventProcessor {
  constructor(event, context, config) {
    this.event = event;
    this.context = context;
    this.apiConfig = config;
  }
  async processEvent(execution, doNotDecodeMessage) {
    const resp = await this._processRawEvent(execution, doNotDecodeMessage);
    if (resp && resp.getCode() != 200) throw new Error(JSON.stringify(resp.getBody()));
    else if (resp) return resp;
  }
  async _processRawEvent(execution, doNotDecodeMessage) {
    if (this.event.Records && this.event.Records.length > 0) { //safe check for empty events?
      //init transaction for all records
      return await (new IKTransaction(this.event, this.context, this.apiConfig, true /*on queue*/)).execute(async (transaction) => {
        //for each available event
        for (let eventRecord of this.event.Records) {
          //Get event content
          const parsedEventRecord = (doNotDecodeMessage ? eventRecord.body : JSON.parse(eventRecord.body));
          //Call execution
          let resp = await execution(transaction, parsedEventRecord);
          if (resp.getCode() != 200) { //if execution failed, respond and mark as answered
            return resp;
          }
        }
        //not errored and loop ended - succeeded
        return IKSuccessResponse();
      });
    } else return IKBadRequestResponse(IKGlobals.ErrorResponseNoRecords, IKGlobals.ErrorCode_NoRecords); //no event to be processed?
  }
}
